/*
  File name        : child_control.h
  Date of creation : 15/11/2008
  Version          : 1.0
  Copyright        : Soft'n'design
  Author           : Laurent Mazet <mazet@softndesign.org>

  Description      : This file contains mechanism to control child processes

  History          :
  - initial version
*/

#ifndef __CHILD_CONTROL__
#define __CHILD_CONTROL__

#include <sys/types.h>

__BEGIN_DECLS

/**
   Register a new child

   @param child_id child identificator
   @param preinit prefork init function
   @param postinit postfork init function
   @param process child process function
   @param term  child termincation function
   @param data intern child data

   @return child pid or 0 on error
*/
pid_t register_child (int child_id, int (*preinit) (void *data), int (*postinit) (void *data), int (*process) (void *data), int (*term) (void *data), void *data);

/**
   Kill all children by a specific signal

   @param cause if different to 0, a log message is displayed before
   sending signal to child processes
   @param sig signal to be send
*/
void kill_all_children (int cause, int sig);

__END_DECLS

#endif /* __CHILD_CONTROL__ */
