/*
  File name        : childcontrol.c
  Date of creation : 2/4/2013
  Version          : 1.0
  Copyright        : Soft'n'design
  Author           : Laurent Mazet <mazet@softndesign.org>

  Description      : This file contains tests for child control

  History          :
  - initial version
*/

/* depend: */
/* cflags: */
/* linker: -lchildcontrol -lm */

#include <assert.h>
#include <libgen.h>
#include <math.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>

#include <snd/childcontrol.h>
#include <snd/childcontrol-verbose.h>

int nb_loops = 10;
int nb_children = 1;

static char *progname = NULL;

typedef struct {
    int id;
} data_t;

int child_process (void *data)
{

    data_t *d = data;

    int delay = (d->id + 1) * 1000;
    float r;

    printf ("TEST: Start child %d.\n", d->id);

    int i;
    for (i = 0; i < nb_loops; i++) {

        if (1) {

            printf ("TEST: Child %d is working (loop %d/%d)\n", d->id, i + 1, nb_loops);

            int j;
            r = 0;
            for (j = 0; j < 100 * d->id; j++) {
                float x = 0;
                do
                    x = (float) rand () / ((float)RAND_MAX + 1);
                while (x == 0);
                float y = (float) rand () / ((float)RAND_MAX + 1);
                float z = sqrtf (-2. * logf (x)) * cosf (2 * M_PI * y);
                r += z * (1 << 31);

                usleep (1000);
            }

        } else {

            printf ("TEST: Child %d is sleeping (%dms. loop %d/%d)\n", d->id, delay / 1000, i + 1, nb_loops);
            usleep (delay);

        }

    }
    
    kill (getpid (), SIGTERM);

    return 0;
}


int child_term (void *data)
{

    data_t *d = data;

    printf ("TEST: Child %d is dieing.\n", d->id);

    return 0;
}

void usage (int rc)
{
    printf ("usage: %s [-c int] [-h] [-l int] [-v int]\n", progname);
    exit (rc);
}

int main (int argc, char *argv[])
{
    progname = basename (strdup (argv[0]));

    /* process arguments */
    char c;
    while ((c = getopt (argc, argv, "c:hl:v:")) != EOF) {
        switch (c) {
        case 'c':
            nb_children = atoi (optarg);
            break;
        case 'h':
            usage (c != 'h');
            break;
        case 'l':
            nb_loops = atoi (optarg);
            break;
        case 'v':
            CHANGE_VERBOSE_LEVEL (child_control, atoi (optarg));
            break;
        }
    }
    if (argc - optind != 0) {
        fprintf (stderr, "%s: invalid option -- %s\n", progname, argv[optind]);
        usage (1);
    }

    int i;
    data_t *d = (data_t *) calloc (sizeof (data_t), nb_children);
    assert (d);
    for (i = 0; i < nb_children; i++) {
        (d + i)->id = i + 1;
        register_child (i + 1, NULL, NULL, child_process, child_term, (void *)(d + i));
    }

    sleep (100);
    free (d);

    return 0;
}

/* test: CHILD_CONTROL_VERBOSE_=2 {} */
/* test: {} -c 4 */

/* vi:set tabstop=4 expandtab shiftwidth=4: this line set vi mode */
